@@ -21,28 +21,26 @@ import java.io.IOException;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.List;
+import java.util.concurrent.TimeUnit;
 
+import org.apache.calcite.plan.RelOptCluster;
 import org.apache.calcite.plan.RelOptPlanner;
+import org.apache.calcite.plan.RelOptRule;
 import org.apache.calcite.plan.RelOptUtil;
 import org.apache.calcite.plan.RelTraitSet;
 import org.apache.calcite.plan.hep.HepMatchOrder;
 import org.apache.calcite.plan.hep.HepPlanner;
-import org.apache.calcite.plan.hep.HepProgram;
 import org.apache.calcite.plan.hep.HepProgramBuilder;
+import org.apache.calcite.plan.volcano.VolcanoPlanner;
 import org.apache.calcite.rel.RelNode;
 import org.apache.calcite.rel.RelShuttleImpl;
-import org.apache.calcite.rel.core.Join;
 import org.apache.calcite.rel.core.Project;
-import org.apache.calcite.rel.core.RelFactories;
 import org.apache.calcite.rel.core.TableFunctionScan;
 import org.apache.calcite.rel.core.TableScan;
 import org.apache.calcite.rel.logical.LogicalValues;
 import org.apache.calcite.rel.metadata.CachingRelMetadataProvider;
 import org.apache.calcite.rel.metadata.ChainedRelMetadataProvider;
 import org.apache.calcite.rel.metadata.RelMetadataProvider;
-import org.apache.calcite.rel.rules.JoinToMultiJoinRule;
-import org.apache.calcite.rel.rules.LoptOptimizeJoinRule;
-import org.apache.calcite.rel.rules.ProjectRemoveRule;
 import org.apache.calcite.rel.type.RelDataType;
 import org.apache.calcite.rex.RexBuilder;
 import org.apache.calcite.rex.RexNode;
@@ -51,8 +49,10 @@ import org.apache.calcite.sql.SqlExplainLevel;
 import org.apache.calcite.sql.SqlNode;
 import org.apache.calcite.sql.TypedSqlNode;
 import org.apache.calcite.sql.validate.SqlValidatorUtil;
-import org.apache.calcite.tools.Planner;
+import org.apache.calcite.tools.Program;
+import org.apache.calcite.tools.Programs;
 import org.apache.calcite.tools.RelConversionException;
+import org.apache.calcite.tools.RuleSet;
 import org.apache.calcite.tools.ValidationException;
 import org.apache.drill.common.JSONOptions;
 import org.apache.drill.common.logical.PlanProperties;
@@ -65,12 +65,12 @@ import org.apache.drill.exec.physical.PhysicalPlan;
 import org.apache.drill.exec.physical.base.AbstractPhysicalVisitor;
 import org.apache.drill.exec.physical.base.PhysicalOperator;
 import org.apache.drill.exec.physical.impl.join.JoinUtils;
+import org.apache.drill.exec.planner.PlannerPhase;
+import org.apache.drill.exec.planner.PlannerType;
 import org.apache.drill.exec.planner.common.DrillRelOptUtil;
 import org.apache.drill.exec.planner.cost.DrillDefaultRelMetadataProvider;
-import org.apache.drill.exec.planner.logical.DrillJoinRel;
 import org.apache.drill.exec.planner.logical.DrillProjectRel;
 import org.apache.drill.exec.planner.logical.DrillRel;
-import org.apache.drill.exec.planner.logical.DrillRelFactories;
 import org.apache.drill.exec.planner.logical.DrillScreenRel;
 import org.apache.drill.exec.planner.logical.DrillStoreRel;
 import org.apache.drill.exec.planner.logical.PreProcessLogicalRel;
@@ -91,7 +91,6 @@ import org.apache.drill.exec.planner.physical.visitor.SelectionVectorPrelVisitor
 import org.apache.drill.exec.planner.physical.visitor.SplitUpComplexExpressions;
 import org.apache.drill.exec.planner.physical.visitor.StarColumnConverter;
 import org.apache.drill.exec.planner.physical.visitor.SwapHashJoinVisitor;
-import org.apache.drill.exec.planner.sql.DrillSqlWorker;
 import org.apache.drill.exec.planner.sql.parser.UnsupportedOperatorsVisitor;
 import org.apache.drill.exec.server.options.OptionManager;
 import org.apache.drill.exec.server.options.OptionValue;
@@ -103,17 +102,17 @@ import org.slf4j.Logger;
 
 import com.fasterxml.jackson.core.JsonProcessingException;
 import com.google.common.base.Preconditions;
+import com.google.common.base.Stopwatch;
 import com.google.common.collect.Lists;
 
 public class DefaultSqlHandler extends AbstractSqlHandler {
   private static final org.slf4j.Logger logger = org.slf4j.LoggerFactory.getLogger(DefaultSqlHandler.class);
 
+  // protected final QueryContext context;
+  private final Pointer<String> textPlan;
+  private final long targetSliceSize;
   protected final SqlHandlerConfig config;
   protected final QueryContext context;
-  protected final HepPlanner hepPlanner;
-  protected final Planner planner;
-  private Pointer<String> textPlan;
-  private final long targetSliceSize;
 
   public DefaultSqlHandler(SqlHandlerConfig config) {
     this(config, null);
@@ -121,53 +120,57 @@ public class DefaultSqlHandler extends AbstractSqlHandler {
 
   public DefaultSqlHandler(SqlHandlerConfig config, Pointer<String> textPlan) {
     super();
-    this.planner = config.getPlanner();
-    this.context = config.getContext();
-    this.hepPlanner = config.getHepPlanner();
     this.config = config;
+    this.context = config.getContext();
     this.textPlan = textPlan;
-    targetSliceSize = context.getOptions().getOption(ExecConstants.SLICE_TARGET).num_val;
+    this.targetSliceSize = config.getContext().getOptions().getOption(ExecConstants.SLICE_TARGET_OPTION);
+
   }
 
-  protected static void log(final String name, final RelNode node, final Logger logger) {
+  protected void log(final PlannerType plannerType, final PlannerPhase phase, final RelNode node, final Logger logger,
+      Stopwatch watch) {
     if (logger.isDebugEnabled()) {
-      logger.debug(name + " : \n" + RelOptUtil.toString(node, SqlExplainLevel.ALL_ATTRIBUTES));
+      log(plannerType.name() + ":" + phase.description, node, logger, watch);
     }
   }
 
-  protected void log(final String name, final Prel node, final Logger logger) {
-    String plan = PrelSequencer.printWithIds(node, SqlExplainLevel.ALL_ATTRIBUTES);
-    if(textPlan != null){
+  protected void log(final String description, final RelNode node, final Logger logger, Stopwatch watch) {
+    if (logger.isDebugEnabled()) {
+      final String plan = RelOptUtil.toString(node, SqlExplainLevel.ALL_ATTRIBUTES);
+      final String time = watch == null ? "" : String.format(" (%dms)", watch.elapsed(TimeUnit.MILLISECONDS));
+      logger.debug(String.format("%s%s:\n%s", description, time, plan));
+    }
+  }
+
+  protected void logAndSetTextPlan(final String description, final Prel prel, final Logger logger) {
+    final String plan = PrelSequencer.printWithIds(prel, SqlExplainLevel.ALL_ATTRIBUTES);
+    if (textPlan != null) {
       textPlan.value = plan;
     }
 
     if (logger.isDebugEnabled()) {
-      logger.debug(name + " : \n" + plan);
+      logger.debug(String.format("%s:\n%s", description, plan));
     }
   }
 
   protected void log(final String name, final PhysicalPlan plan, final Logger logger) throws JsonProcessingException {
     if (logger.isDebugEnabled()) {
-      String planText = plan.unparse(context.getConfig().getMapper().writer());
+      String planText = plan.unparse(context.getLpPersistence().getMapper().writer());
       logger.debug(name + " : \n" + planText);
     }
   }
 
-
   @Override
   public PhysicalPlan getPlan(SqlNode sqlNode) throws ValidationException, RelConversionException, IOException, ForemanSetupException {
     final ConvertedRelNode convertedRelNode = validateAndConvert(sqlNode);
     final RelDataType validatedRowType = convertedRelNode.getValidatedRowType();
     final RelNode queryRelNode = convertedRelNode.getConvertedNode();
 
-    log("Optiq Logical", queryRelNode, logger);
-    DrillRel drel = convertToDrel(queryRelNode, validatedRowType);
-
-    log("Drill Logical", drel, logger);
-    Prel prel = convertToPrel(drel);
-    log("Drill Physical", prel, logger);
-    PhysicalOperator pop = convertToPop(prel);
-    PhysicalPlan plan = convertToPlan(pop);
+    final DrillRel drel = convertToDrel(queryRelNode, validatedRowType);
+    final Prel prel = convertToPrel(drel);
+    logAndSetTextPlan("Drill Physical", prel, logger);
+    final PhysicalOperator pop = convertToPop(prel);
+    final PhysicalPlan plan = convertToPlan(pop);
     log("Drill Plan", plan, logger);
     return plan;
   }
@@ -203,29 +206,77 @@ public class DefaultSqlHandler extends AbstractSqlHandler {
    * @throws SqlUnsupportedException
    * @throws RelConversionException
    */
-  protected DrillRel convertToDrel(RelNode relNode) throws SqlUnsupportedException, RelConversionException {
+  protected DrillRel convertToDrel(final RelNode relNode) throws SqlUnsupportedException, RelConversionException {
+    // Check if this is a "limit 0 " query.
+    final boolean limitZero = FindLimit0Visitor.containsLimit0(relNode);
+    if (context.getOptions().getOption(ExecConstants.EARLY_LIMIT0_OPT) &&
+        context.getPlannerSettings().isTypeInferenceEnabled() &&
+        limitZero) {
+      // disable distributed mode
+      context.getPlannerSettings().forceSingleMode();
+      // if the schema is known, return the schema directly
+      final DrillRel shorterPlan;
+      if ((shorterPlan = FindLimit0Visitor.getDirectScanRelIfFullySchemaed(relNode)) != null) {
+        return shorterPlan;
+      }
+    }
+
     try {
-      final DrillRel convertedRelNode;
+      final RelNode convertedRelNode;
 
-      final boolean limitZero = context.getPlannerSettings().isLimitZeroOptEnabled() && FindLimit0Visitor.containsLimit0(relNode);
+      PlannerPhase volPlannerPhase = null;
+
+      // HEP Directory pruning .
+      final RelNode pruned = transform(PlannerType.HEP_BOTTOM_UP, PlannerPhase.DIRECTORY_PRUNING, relNode);
+      final RelTraitSet logicalTraits = pruned.getTraitSet().plus(DrillRel.DRILL_LOGICAL);
+
+      if (!context.getPlannerSettings().isHepOptEnabled()) {
+        // Disable project pushdown for limit 0 query.
+        volPlannerPhase = limitZero ? PlannerPhase.LOGICAL_PRUNE_AND_JOIN_NOPROJPD : PlannerPhase.LOGICAL_PRUNE_AND_JOIN;
+
+        // hep is disabled, use volcano
+        convertedRelNode = transform(PlannerType.VOLCANO, volPlannerPhase, pruned, logicalTraits);
 
-      if (! context.getPlannerSettings().isHepJoinOptEnabled()) {
-        convertedRelNode = (DrillRel) logicalPlanningVolcano(relNode, limitZero);
       } else {
-        convertedRelNode = (DrillRel) logicalPlanningVolcanoAndLopt(relNode, limitZero);
+        final RelNode intermediateNode2;
+        if (context.getPlannerSettings().isHepPartitionPruningEnabled()) {
+          // Disable project pushdown for .
+          volPlannerPhase = limitZero ? PlannerPhase.LOGICAL_NOPROJPD : PlannerPhase.LOGICAL;
+
+          // hep is enabled and hep pruning is enabled.
+          final RelNode intermediateNode = transform(PlannerType.VOLCANO, volPlannerPhase, pruned, logicalTraits);
+          intermediateNode2 = transform(PlannerType.HEP_BOTTOM_UP, PlannerPhase.PARTITION_PRUNING, intermediateNode);
+
+        } else {
+          // Disable project pushdown for .
+          volPlannerPhase = limitZero ? PlannerPhase.LOGICAL_PRUNE_NOPROJPD : PlannerPhase.LOGICAL_PRUNE;
+
+          // Only hep is enabled
+          intermediateNode2 = transform(PlannerType.VOLCANO, volPlannerPhase, pruned, logicalTraits);
+        }
+
+        // Do Join Planning.
+        convertedRelNode = transform(PlannerType.HEP_BOTTOM_UP, PlannerPhase.JOIN_PLANNING, intermediateNode2);
       }
 
-      if (convertedRelNode instanceof DrillStoreRel) {
+      // Convert SUM to $SUM0
+      final RelNode convertedRelNodeWithSum0 = transform(PlannerType.HEP_BOTTOM_UP, PlannerPhase.SUM_CONVERSION, convertedRelNode);
+
+      final DrillRel drillRel = (DrillRel) convertedRelNodeWithSum0;
+
+      if (drillRel instanceof DrillStoreRel) {
         throw new UnsupportedOperationException();
       } else {
 
         // If the query contains a limit 0 clause, disable distributed mode since it is overkill for determining schema.
-        if (FindLimit0Visitor.containsLimit0(convertedRelNode)) {
+        if (FindLimit0Visitor.containsLimit0(convertedRelNodeWithSum0)) {
           context.getPlannerSettings().forceSingleMode();
-          return FindLimit0Visitor.addLimitOnTopOfLeafNodes(convertedRelNode);
+          if (context.getOptions().getOption(ExecConstants.LATE_LIMIT0_OPT)) {
+            return FindLimit0Visitor.addLimitOnTopOfLeafNodes(drillRel);
+          }
         }
 
-        return convertedRelNode;
+        return drillRel;
       }
     } catch (RelOptPlanner.CannotPlanException ex) {
       logger.error(ex.getMessage());
@@ -251,7 +302,7 @@ public class DefaultSqlHandler extends AbstractSqlHandler {
     final DrillRel convertedRelNode = convertToDrel(relNode);
 
     // Put a non-trivial topProject to ensure the final output field name is preserved, when necessary.
-    DrillRel topPreservedNameProj = addRenamedProject((DrillRel) convertedRelNode, validatedRowType);
+    DrillRel topPreservedNameProj = addRenamedProject(convertedRelNode, validatedRowType);
     return new DrillScreenRel(topPreservedNameProj.getCluster(), topPreservedNameProj.getTraitSet(),
         topPreservedNameProj);
   }
@@ -272,13 +323,124 @@ public class DefaultSqlHandler extends AbstractSqlHandler {
 
   }
 
+  /**
+   * Transform RelNode to a new RelNode without changing any traits. Also will log the outcome.
+   *
+   * @param plannerType
+   *          The type of Planner to use.
+   * @param phase
+   *          The transformation phase we're running.
+   * @param input
+   *          The origianl RelNode
+   * @return The transformed relnode.
+   */
+  private RelNode transform(PlannerType plannerType, PlannerPhase phase, RelNode input) {
+    return transform(plannerType, phase, input, input.getTraitSet());
+  }
+
+  /**
+   * Transform RelNode to a new RelNode, targeting the provided set of traits. Also will log the outcome.
+   *
+   * @param plannerType
+   *          The type of Planner to use.
+   * @param phase
+   *          The transformation phase we're running.
+   * @param input
+   *          The origianl RelNode
+   * @param targetTraits
+   *          The traits we are targeting for output.
+   * @return The transformed relnode.
+   */
+  protected RelNode transform(PlannerType plannerType, PlannerPhase phase, RelNode input, RelTraitSet targetTraits) {
+    return transform(plannerType, phase, input, targetTraits, true);
+  }
+
+  /**
+   * Transform RelNode to a new RelNode, targeting the provided set of traits. Also will log the outcome if asked.
+   *
+   * @param plannerType
+   *          The type of Planner to use.
+   * @param phase
+   *          The transformation phase we're running.
+   * @param input
+   *          The origianl RelNode
+   * @param targetTraits
+   *          The traits we are targeting for output.
+   * @param log
+   *          Whether to log the planning phase.
+   * @return The transformed relnode.
+   */
+  protected RelNode transform(PlannerType plannerType, PlannerPhase phase, RelNode input, RelTraitSet targetTraits,
+      boolean log) {
+    final Stopwatch watch = Stopwatch.createStarted();
+    final RuleSet rules = config.getRules(phase);
+    final RelTraitSet toTraits = targetTraits.simplify();
+
+    final RelNode output;
+    switch (plannerType) {
+    case HEP_BOTTOM_UP:
+    case HEP: {
+      final HepProgramBuilder hepPgmBldr = new HepProgramBuilder();
+      if (plannerType == PlannerType.HEP_BOTTOM_UP) {
+        hepPgmBldr.addMatchOrder(HepMatchOrder.BOTTOM_UP);
+      }
+      for (RelOptRule rule : rules) {
+        hepPgmBldr.addRuleInstance(rule);
+      }
+
+      final HepPlanner planner = new HepPlanner(hepPgmBldr.build(), context.getPlannerSettings());
+
+      final List<RelMetadataProvider> list = Lists.newArrayList();
+      list.add(DrillDefaultRelMetadataProvider.INSTANCE);
+      planner.registerMetadataProviders(list);
+      final RelMetadataProvider cachingMetaDataProvider = new CachingRelMetadataProvider(
+          ChainedRelMetadataProvider.of(list), planner);
+
+      // Save original metadataProvider before call HepPlanner call.
+      RelMetadataProvider oldProvider = input.getCluster().getMetadataProvider();
+      input.getCluster().setMetadataProvider(cachingMetaDataProvider);
+      planner.setRoot(input);
+      if (!input.getTraitSet().equals(targetTraits)) {
+        planner.changeTraits(input, toTraits);
+      }
+      output = planner.findBestExp();
+
+      // Restore to original metadataProvider after HepPlanner call.
+      input.getCluster().setMetadataProvider(oldProvider);
+      break;
+    }
+    case VOLCANO:
+    default: {
+      // as weird as it seems, the cluster's only planner is the volcano planner.
+      final RelOptPlanner planner = input.getCluster().getPlanner();
+      final Program program = Programs.of(rules);
+      Preconditions.checkArgument(planner instanceof VolcanoPlanner,
+          "Cluster is expected to be constructed using VolcanoPlanner. Was actually of type %s.", planner.getClass()
+              .getName());
+      output = program.run(planner, input, toTraits);
+
+      break;
+    }
+    }
+
+    if (log) {
+      log(plannerType, phase, output, logger, watch);
+    }
+
+    return output;
+  }
+
   protected Prel convertToPrel(RelNode drel) throws RelConversionException, SqlUnsupportedException {
     Preconditions.checkArgument(drel.getConvention() == DrillRel.DRILL_LOGICAL);
-    RelTraitSet traits = drel.getTraitSet().plus(Prel.DRILL_PHYSICAL).plus(DrillDistributionTrait.SINGLETON);
+
+    final RelTraitSet traits = drel.getTraitSet().plus(Prel.DRILL_PHYSICAL).plus(DrillDistributionTrait.SINGLETON);
     Prel phyRelNode;
     try {
-      final RelNode relNode = planner.transform(DrillSqlWorker.PHYSICAL_MEM_RULES, traits, drel);
+      final Stopwatch watch = Stopwatch.createStarted();
+      final RelNode relNode = transform(PlannerType.VOLCANO, PlannerPhase.PHYSICAL, drel, traits, false);
       phyRelNode = (Prel) relNode.accept(new PrelFinalizer());
+      // log externally as we need to finalize before traversing the tree.
+      log(PlannerType.VOLCANO, PlannerPhase.PHYSICAL, phyRelNode, logger, watch);
     } catch (RelOptPlanner.CannotPlanException ex) {
       logger.error(ex.getMessage());
 
@@ -292,15 +454,15 @@ public class DefaultSqlHandler extends AbstractSqlHandler {
     OptionManager queryOptions = context.getOptions();
 
     if (context.getPlannerSettings().isMemoryEstimationEnabled()
-      && !MemoryEstimationVisitor.enoughMemory(phyRelNode, queryOptions, context.getActiveEndpoints().size())) {
-      log("Not enough memory for this plan", phyRelNode, logger);
+        && !MemoryEstimationVisitor.enoughMemory(phyRelNode, queryOptions, context.getActiveEndpoints().size())) {
+      log("Not enough memory for this plan", phyRelNode, logger, null);
       logger.debug("Re-planning without hash operations.");
 
       queryOptions.setOption(OptionValue.createBoolean(OptionValue.OptionType.QUERY, PlannerSettings.HASHJOIN.getOptionName(), false));
       queryOptions.setOption(OptionValue.createBoolean(OptionValue.OptionType.QUERY, PlannerSettings.HASHAGG.getOptionName(), false));
 
       try {
-        final RelNode relNode = planner.transform(DrillSqlWorker.PHYSICAL_MEM_RULES, traits, drel);
+        final RelNode relNode = transform(PlannerType.VOLCANO, PlannerPhase.PHYSICAL, drel, traits);
         phyRelNode = (Prel) relNode.accept(new PrelFinalizer());
       } catch (RelOptPlanner.CannotPlanException ex) {
         logger.error(ex.getMessage());
@@ -313,7 +475,7 @@ public class DefaultSqlHandler extends AbstractSqlHandler {
       }
     }
 
-    /*  The order of the following transformation is important */
+    /* The order of the following transformations is important */
 
     /*
      * 0.) For select * from join query, we need insert project on top of scan and a top project just
@@ -336,18 +498,22 @@ public class DefaultSqlHandler extends AbstractSqlHandler {
      * We want to have smaller dataset on the right side, since hash table builds on right side.
      */
     if (context.getPlannerSettings().isHashJoinSwapEnabled()) {
-      phyRelNode = SwapHashJoinVisitor.swapHashJoin(phyRelNode, new Double(context.getPlannerSettings().getHashJoinSwapMarginFactor()));
+      phyRelNode = SwapHashJoinVisitor.swapHashJoin(phyRelNode, new Double(context.getPlannerSettings()
+          .getHashJoinSwapMarginFactor()));
     }
 
     /*
      * 1.2) Break up all expressions with complex outputs into their own project operations
      */
-    phyRelNode = ((Prel) phyRelNode).accept(new SplitUpComplexExpressions(planner.getTypeFactory(), context.getDrillOperatorTable(), context.getPlannerSettings().functionImplementationRegistry), null);
+    phyRelNode = phyRelNode.accept(
+        new SplitUpComplexExpressions(config.getConverter().getTypeFactory(), context.getDrillOperatorTable(), context
+            .getPlannerSettings().functionImplementationRegistry), null);
 
     /*
      * 1.3) Projections that contain reference to flatten are rewritten as Flatten operators followed by Project
      */
-    phyRelNode = ((Prel) phyRelNode).accept(new RewriteProjectToFlatten(planner.getTypeFactory(), context.getDrillOperatorTable()), null);
+    phyRelNode = phyRelNode.accept(
+        new RewriteProjectToFlatten(config.getConverter().getTypeFactory(), context.getDrillOperatorTable()), null);
 
     /*
      * 2.)
@@ -447,9 +613,9 @@ public class DefaultSqlHandler extends AbstractSqlHandler {
   }
 
   private TypedSqlNode validateNode(SqlNode sqlNode) throws ValidationException, RelConversionException, ForemanSetupException {
-    TypedSqlNode typedSqlNode = planner.validateAndGetType(sqlNode);
-
-    SqlNode sqlNodeValidated = typedSqlNode.getSqlNode();
+    final SqlNode sqlNodeValidated = config.getConverter().validate(sqlNode);
+    final TypedSqlNode typedSqlNode = new TypedSqlNode(sqlNodeValidated, config.getConverter().getOutputType(
+        sqlNodeValidated));
 
     // Check if the unsupported functionality is used
     UnsupportedOperatorsVisitor visitor = UnsupportedOperatorsVisitor.createVisitor(context);
@@ -467,22 +633,9 @@ public class DefaultSqlHandler extends AbstractSqlHandler {
   }
 
   private RelNode convertToRel(SqlNode node) throws RelConversionException {
-    final RelNode convertedNode = planner.convert(node);
-
-    final RelMetadataProvider provider = convertedNode.getCluster().getMetadataProvider();
-
-    // Register RelMetadataProvider with HepPlanner.
-    final List<RelMetadataProvider> list = Lists.newArrayList(provider);
-    hepPlanner.registerMetadataProviders(list);
-    final RelMetadataProvider cachingMetaDataProvider = new CachingRelMetadataProvider(ChainedRelMetadataProvider.of(list), hepPlanner);
-    convertedNode.accept(new MetaDataProviderModifier(cachingMetaDataProvider));
-
-    // HepPlanner is specifically used for Window Function planning only.
-    hepPlanner.setRoot(convertedNode);
-    RelNode rel = hepPlanner.findBestExp();
-
-    rel.accept(new MetaDataProviderModifier(provider));
-    return rel;
+    final RelNode convertedNode = config.getConverter().toRel(node);
+    log("INITIAL", convertedNode, logger, null);
+    return transform(PlannerType.HEP, PlannerPhase.WINDOW_REWRITE, convertedNode);
   }
 
   private RelNode preprocessNode(RelNode rel) throws SqlUnsupportedException {
@@ -494,8 +647,9 @@ public class DefaultSqlHandler extends AbstractSqlHandler {
      * 2. see where the tree contains unsupported functions; throw SqlUnsupportedException if there is any.
      */
 
-    PreProcessLogicalRel visitor = PreProcessLogicalRel.createVisitor(planner.getTypeFactory(),
-        context.getDrillOperatorTable());
+    PreProcessLogicalRel visitor = PreProcessLogicalRel.createVisitor(config.getConverter().getTypeFactory(),
+        context.getDrillOperatorTable(),
+        rel.getCluster().getRexBuilder());
     try {
       rel = rel.accept(visitor);
     } catch (UnsupportedOperationException ex) {
@@ -531,76 +685,6 @@ public class DefaultSqlHandler extends AbstractSqlHandler {
     }
   }
 
-
-  private RelNode logicalPlanningVolcano(RelNode relNode, boolean limitZero) throws RelConversionException, SqlUnsupportedException {
-    final int ruleIndex = limitZero? DrillSqlWorker.LOGICAL_LIMIT0_RULES : DrillSqlWorker.LOGICAL_RULES;
-
-    return planner.transform(ruleIndex, relNode.getTraitSet().plus(DrillRel.DRILL_LOGICAL), relNode);
-  }
-
-  /**
-   * Do logical planning using both VolcanoPlanner and LOPT HepPlanner.
-   * @param relNode
-   * @return
-   * @throws RelConversionException
-   * @throws SqlUnsupportedException
-   */
-  private RelNode logicalPlanningVolcanoAndLopt(RelNode relNode, boolean limitZero) throws RelConversionException, SqlUnsupportedException {
-    final int ruleIndex = limitZero ? DrillSqlWorker.LOGICAL_LOPT_LIMIT0_RULES : DrillSqlWorker.LOGICAL_LOPT_RULES;
-
-    final RelNode convertedRelNode = planner.transform(ruleIndex, relNode.getTraitSet().plus(DrillRel.DRILL_LOGICAL), relNode);
-    log("VolCalciteRel", convertedRelNode, logger);
-
-    final RelNode loptNode = getLoptJoinOrderTree(
-        convertedRelNode,
-        DrillJoinRel.class,
-        DrillRelFactories.DRILL_LOGICAL_JOIN_FACTORY,
-        DrillRelFactories.DRILL_LOGICAL_FILTER_FACTORY,
-        DrillRelFactories.DRILL_LOGICAL_PROJECT_FACTORY);
-
-    log("HepCalciteRel", loptNode, logger);
-
-    return loptNode;
-  }
-
-
-  /**
-   * Appy Join Order Optimizations using Hep Planner.
-   */
-  private RelNode getLoptJoinOrderTree(RelNode root,
-                                      Class<? extends Join> joinClass,
-                                             RelFactories.JoinFactory joinFactory,
-                                             RelFactories.FilterFactory filterFactory,
-                                             RelFactories.ProjectFactory projectFactory) {
-    final HepProgramBuilder hepPgmBldr = new HepProgramBuilder()
-        .addMatchOrder(HepMatchOrder.BOTTOM_UP)
-        .addRuleInstance(new JoinToMultiJoinRule(joinClass))
-        .addRuleInstance(new LoptOptimizeJoinRule(joinFactory, projectFactory, filterFactory))
-        .addRuleInstance(ProjectRemoveRule.INSTANCE);
-        // .addRuleInstance(new ProjectMergeRule(true, projectFactory));
-
-        // .addRuleInstance(DrillMergeProjectRule.getInstance(true, projectFactory, this.context.getFunctionRegistry()));
-
-
-    final HepProgram hepPgm = hepPgmBldr.build();
-    final HepPlanner hepPlanner = new HepPlanner(hepPgm);
-
-    final List<RelMetadataProvider> list = Lists.newArrayList();
-    list.add(DrillDefaultRelMetadataProvider.INSTANCE);
-    hepPlanner.registerMetadataProviders(list);
-    final RelMetadataProvider cachingMetaDataProvider = new CachingRelMetadataProvider(ChainedRelMetadataProvider.of(list), hepPlanner);
-
-    // Modify RelMetaProvider for every RelNode in the SQL operator Rel tree.
-    root.accept(new MetaDataProviderModifier(cachingMetaDataProvider));
-
-    hepPlanner.setRoot(root);
-
-    RelNode calciteOptimizedPlan = hepPlanner.findBestExp();
-
-    return calciteOptimizedPlan;
-  }
-
-
   public static class MetaDataProviderModifier extends RelShuttleImpl {
     private final RelMetadataProvider metadataProvider;
 
